"""
    Archetypes items used in DataGridField unit testing and examples
    
"""
# Zope imports
from AccessControl import ClassSecurityInfo

# Plone imports
from Products.Archetypes.public import DisplayList
from Products.Archetypes.public import *

# Local imports
from Products.DataGridField import DataGridField, DataGridWidget
from Products.DataGridField.Column import Column
from Products.DataGridField.SelectColumn import SelectColumn
from Products.DataGridField.RadioColumn import RadioColumn
from Products.DataGridField.FixedColumn import FixedColumn
from Products.DataGridField.DataGridField import FixedRow

class DataGridDemoType(BaseContent):
    """ Demo for based DataGridField functionality
    
    This class is used in unit testing
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((

        DataGridField('DemoField',
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),                
            ),
        ),                                 
    )
                                                                      
    meta_type = portal_type = archetype_name = 'DataGridDemoType'
    
registerType(DataGridDemoType)    

class DataGridDemoType2(BaseContent):
    """ Demo for different DataGridWidget columns
    
    This class is used in unit testing
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((
        DataGridField('DemoField',
                searchable = True,
                columns=("column1", "column2", "select_sample"),
                widget = DataGridWidget(
                    columns={
                        'column1' : Column("Toholampi city rox"),
                        'column2' : Column("My friendly name"),
                        'select_sample' : SelectColumn("Friendly name", vocabulary="getSampleVocabulary")
                    },
                 ),                 
         ),
                                  
        DataGridField('DemoField2',
                searchable = True,
                columns=("column1", "column2", "select_sample"),
                widget = DataGridWidget(
                    columns={
                        'column1' : Column("Toholampi city rox"),
                        'column2' : Column("My friendly name"),
                        'select_sample' : RadioColumn("Friendly name", vocabulary="getSampleVocabulary")
                    },
                 ),                 
         ),                                  
                
        ))
    
    meta_type = portal_type = archetype_name = 'DataGridDemoType2'
    
    def getSampleVocabulary(self):
        """Get a sample vocabulary
        """
        return DisplayList(
    
            (("sample", "Sample value 1",),
            ("sample2", "Sample value 2",),))

registerType(DataGridDemoType2)

class InvalidDataGridDemoType(BaseContent):
    """ DataGridField declaration with errors
    
    Errors should be detected run-time, with helpful error messages.
    
    This class is missing column definition select_sample in DataGridWidget        
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((
        DataGridField('DemoField',
                searchable = True,
                columns=("column1", "column2", "select_sample"),
                widget = DataGridWidget(
                    columns={
                        'column1' : Column("Toholampi city rox"),
                        'column2' : Column("My friendly name"),
                    },                 
          ),),
                
        ))
    
    meta_type = portal_type = archetype_name = 'InvalidDataGridDemoType'        
    
registerType(InvalidDataGridDemoType)
    
    
class FixedRowsDemoType(BaseContent):
    """ Demostrate fixed rows usage
    
    This class is used in unit testing
    """
    security = ClassSecurityInfo()
    
    schema = BaseSchema + Schema((

        DataGridField('DemoField',
                widget = DataGridWidget(),
                columns=('column1','column2','The third'),
                fixed_rows = [
                    FixedRow(keyColumn="column1", initialData = { "column1" : "must-exist-1", "column2" : "bbb" }),
                    FixedRow(keyColumn="column2", initialData = { "column1" : "ddd", "column2" : "must-exist-2" }),
                ]
            ),
                                  
        DataGridField('predefinedSkills',
            searchable=True,          
            columns=('skill', 'level'),
            fixed_rows = "getPredefinedSkillsData",
            allow_delete = False,
            allow_insert = False,
            allow_reorder = False,
            widget = DataGridWidget(
                label="Skills",
                description="Language/technology/tool/method for which employer has special interest",
                columns= {
                    "skill" : FixedColumn("Skill"),
                    "level" : RadioColumn("Level", vocabulary="getSkillLevels")
                }
            ),            
            ),                                  
        ))
        
    meta_type = portal_type = archetype_name = 'FixedRowsDemoType'
        
    def getSkillLevels(self):
        return DisplayList(    
        (("bad", "Bad",),
         ("good", "Good",),
        ))
        
    def getPredefinedSkillsData(self):      
        """ Generate fixed row key information """
        skills = [ "Python", "Perl", "XML", "Java", "Plone" ]
        rows = []
        for skill in skills:            
            rows.append(FixedRow(keyColumn="skill", initialData={"skill" : skill, "level" : "bad"}))
            
        return rows
    
    
registerType(FixedRowsDemoType)    
    



