#!/bin/bash
#
# A script to build release gzip-archive from a product
# Copyright Mikko Ohtamaa 2005
#
# See release-sh-readme.html for licence & documentation
#
# This script must be run in the add-on product folder!
#

ZOPE_PATH="../../Zope-2.8.6-final"

ZOPE_PRODUCTS=$ZOPEPATH/lib/python/Products

# Optional dependencies packed with this product
EXTRAS="DataGridField AdvancedQuery PloneInstallation"

# Excluded folders and files
# - No compiled python files (with or withour debug information)
# - No Paint Shop Pro thumbnial files
# - No Subversion or CVS version management files
EXCLUDE=("*.pyc" "*.pyd" ".svn" ".cvs" "*.jbf" "screenshots")

if [ "$1" != ""  ] ; then
   # Destination path is the first argument if the argument exists
   DESTINATION_PATH=$1
else
   DESTINATION_PATH=`cd ..; pwd`
fi

# Get product name from the current path (execute pwd command)
PRODUCT=`basename "\`pwd\`"`

# Tarball variable is the base name of the package
# TODO: How to lowercase a variable in Unix shell
TARBALL=$PRODUCT

# Read version from version.txt
# Note that versio.txt must not contain trailing new line
VERSION=`cat version.txt`

# Tar filename and folder including version information
FULL_TARBALL=$DESTINATION_PATH/$TARBALL-$VERSION.tar.gz

# Generate README.txt from Read me.html
if [ -e "Read me.html" ] ; then
	echo Creating plain text README
	python release_tool/html2text.py < "Read me.html" > README.txt
fi

echo Now building release archive $FULL_TARBALL

# Force user access rights
# In Windows (Cygwin) environment there a+rx is missing causing problems when
# a product is transferred to Linux
chmod -R a+rx ../$PRODUCT

# Build tar command line
ARCHIVE_CMD="tar"

# Concatenate exclude mask for tar
index=0
while [ "$index" -lt ${#EXCLUDE[@]} ]; do
    echo $index
    ARCHIVE_CMD=$ARCHIVE_CMD" --exclude=${EXCLUDE[$index]}"
    let "index = $index + 1"
done

# Add rest of tar command
ARCHIVE_CMD=$ARCHIVE_CMD" -cvzf $FULL_TARBALL $PRODUCT $EXTRAS"

# We need to cd (bad in script)
# Otherwise tar doesn't get path name right
cd ..

# Create tar package
echo Creating tar package $ARCHIVE_CMD
$ARCHIVE_CMD

cd $PRODUCT
